package vclipx;

import maspack.matrix.*;

/**
 * Scratch space for storing transformed features components
 * associated with a specific polyhedron.
 *
 * @author <a href="http://www.cs.ubc.ca/~lloyd">John E. Lloyd</a>, Winter 2003
 */
class FeatureBlock
{
	RigidTransform3d T = new RigidTransform3d();

	Feature feat;
	Point3d xcoords = new Point3d();
	Point3d xtail = new Point3d();
	Point3d xhead = new Point3d();
	Vector3d xseg = new Vector3d();

	Point3d pnt = new Point3d();
	Vector3d nrm = new Vector3d();

	Point3d xpnt = new Point3d();
	Vector3d xvec = new Vector3d();

	Edge edge;
	Vertex vert;
	Face face;

	ConvexPolyhedron.DoublePtr dist = new ConvexPolyhedron.DoublePtr(0);

	/* Constructor */
	FeatureBlock (Feature f)
	 { if (f != null)
	    { setFeature (f);
	    }
	 }

	void xformVertex ()
	 {
	   xcoords.transform (T, vert.coords);
	 }

	void xformEdge ()
	 {
	   xtail.transform (T, edge.tail.coords);
	   xhead.transform (T, edge.head.coords);
	   xseg.sub (xhead, xtail);
	 }

	void setFeature (Feature f)
	 {
	   if (f == null)
	    { feat = null;
	      edge = null;
	      vert = null;
	      face = null;
	    }
	   else if (f.type == Feature.VERTEX)
	    { setVert ((Vertex)f);
	    }
	   else if (f.type == Feature.EDGE)
	    { setEdge ((Edge)f);
	    }
	   else if (f.type == Feature.FACE)
	    { setFace ((Face)f);
	    }
	 }

	void setVert (Vertex v)
	 { feat = v;
	   vert = v;
	   xformVertex();
	 }

	void setEdge (Edge e)
	 { feat = e;
	   edge = e;
	   xformEdge();
	 }

	void setFace (Face f)
	 { feat = f;
	   face = f;
	 }
}
